#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

__revision__ = "$Id: conf.py,v 1.112 2004/04/17 16:45:29 jemfinch Exp $"

import fix

import os
import sys
import socket
import string

import utils
import registry
import ircutils

installDir = os.path.dirname(os.path.dirname(sys.modules[__name__].__file__))
_srcDir = os.path.join(installDir, 'src')
_pluginsDir = os.path.join(installDir, 'plugins')

###
# allowEval: True if the owner (and only the owner) should be able to eval
#            arbitrary Python code.  This is specifically *not* a registry
#            variable because it shouldn't be modifiable in the bot.
###
allowEval = False

###
# strictRfc: Determines whether the bot will very strictly follow the RCE
#            or whether it will allow things like @ and . in nicks.
###
strictRfc = False

supybot = registry.Group()
supybot.setName('supybot')

def registerPlugin(name, currentValue=None):
    supybot.plugins.register(name, registry.Boolean(False, """Determines
    whether this plugin is loaded by default.""", showDefault=False))
    if currentValue is not None:
        supybot.plugins.get(name).setValue(currentValue)

def registerChannelValue(group, name, value):
    value.supplyDefault = True
    group.register(name, value)

def registerGlobalValue(group, name, value):
    group.register(name, value)

def registerGroup(Group, name, group=None):
    Group.register(name, group)

class ValidNick(registry.String):
    """Value must be a valid IRC nick."""
    def setValue(self, v):
        if not ircutils.isNick(v):
            self.error()
        else:
            registry.String.setValue(self, v)

class ValidChannel(registry.String):
    """Value must be a valid IRC channel name."""
    def setValue(self, v):
        if ',' in v:
            (channel, _) = v.split(',', 1)
        else:
            channel = v
        if not ircutils.isChannel(channel):
            self.error()
        else:
            registry.String.setValue(self, v)

supybot.register('nick', ValidNick('supybot',
"""Determines the bot's nick."""))

supybot.register('ident', ValidNick('supybot',
"""Determines the bot's ident."""))

supybot.register('user', registry.String('supybot', """Determines the user
the bot sends to the server."""))

# TODO: Make this check for validity.
supybot.register('server', registry.String('irc.freenode.net', """Determines
what server the bot connects to."""))

supybot.register('password', registry.String('', """Determines the password to
be sent to the server if it requires one."""))

class SpaceSeparatedSetOfChannels(registry.SeparatedListOf):
    List = ircutils.IrcSet
    Value = ValidChannel
    def splitter(self, s):
        return s.split()
    joiner = ' '.join

    def removeChannel(self, channel):
        removals = []
        for c in self.value:
            chan = c
            if ',' in c:
                (chan, _) = c.split(',')
            if chan == channel:
                removals.append(c)
        for removal in removals:
            self.value.remove(discard)

supybot.register('channels', SpaceSeparatedSetOfChannels(['#supybot'], """
Determines what channels the bot will join when it connects to the server."""))

class ValidPrefixChars(registry.String):
    """Value must contain only ~!@#$%^&*()_-+=[{}]\\|'\";:,<.>/?"""
    def setValue(self, v):
        if v.translate(string.ascii, '`~!@#$%^&*()_-+=[{}]\\|\'";:,<.>/?'):
            self.error()
        registry.String.setValue(self, v)

supybot.register('prefixChars', ValidPrefixChars('', """Determines what prefix
characters the bot will reply to.  A prefix character is a single character
that the bot will use to determine what messages are addressed to it; when
there are no prefix characters set, it just uses its nick.  Each character in
this string is interpreted individually; you can have multiple prefixChars
simultaneously, and if any one of them is used as a prefix the bot will
assume it is being addressed."""))

supybot.register('defaultCapabilities',
registry.CommaSeparatedSetOfStrings(['-owner', '-admin', '-trusted'], """
These are the capabilities that are given to everyone by default.  If they are
normal capabilities, then the user will have to have the appropriate
anti-capability if you want to override these capabilities; if they are
anti-capabilities, then the user will have to have the actual capability to
override these capabilities.  See docs/CAPABILITIES if you don't understand
why these default to what they do."""))

supybot.register('defaultAllow', registry.Boolean(True, """Determines whether
the bot by default will allow users to run commands.  If this is disabled, a
user will have to have the capability for whatever command he wishes to run.
"""))

supybot.register('defaultIgnore', registry.Boolean(False, """Determines
whether the bot will ignore unregistered users by default.  Of course, that'll
make it particularly hard for those users to register with the bot, but that's
your problem to solve."""))


supybot.register('humanTimestampFormat', registry.String('%I:%M %p, %B %d, %Y',
"""Determines how timestamps printed for human reading should be formatted.
Refer to the Python documentation for the time module to see valid formatting
characteres for time formats."""))

class IP(registry.String):
    """Value must be a valid IP."""
    def setValue(self, v):
        if v and not (utils.isIP(v) or utils.isIPV6(v)):
            self.error()
        else:
            registry.String.setValue(self, v)
        
supybot.register('externalIP', IP('', """A string that is the external IP of
the bot.  If this is the empty string, the bot will attempt to find out its IP
dynamically (though sometimes that doesn't work, hence this variable)."""))

###
# Reply/error tweaking.
###
supybot.register('reply')
supybot.reply.register('truncate', registry.Boolean(False, """Determines
whether the bot will simply truncate messages instead of breaking up long
messages and using the 'more' command to get the remaining chunks."""))
supybot.reply.register('maximumMores', registry.PositiveInteger(50, """
Determines what the maximum number of chunks (for use with the 'more' command)
will be."""))

supybot.reply.register('oneToOne', registry.Boolean(True, """Determines whether
the bot will send multi-message replies in a single messsage or in multiple
messages.  For safety purposes (so the bot can't possibly flood) it will
normally send everything in a single message."""))

supybot.register('bracketSyntax', registry.Boolean(True, """Supybot allows
nested commands. If this option is enabled users can nest commands using a
bracket syntax, for example: 'bot: bar [foo]'."""))

class ValidBrackets(registry.OnlySomeStrings):
    validStrings = ('[]', '<>', '{}', '()')
    
supybot.register('brackets', ValidBrackets('[]', """Supybot allows you to
specify what brackets are used for your nested commands.  Valid sets of
brackets include [], <>, and {} ().  [] has strong historical motivation, as
well as being the brackets that don't require shift.  <> or () might be
slightly superior because they cannot occur in a nick."""))

supybot.register('pipeSyntax', registry.Boolean(False, """Supybot allows
nested commands. Enabling this option will allow nested commands with a syntax
similar to UNIX pipes, for example: 'bot: foo | bar'."""))

supybot.reply.register('whenNotCommand', registry.Boolean(True, """
Determines whether the bot will reply with an error message when it is
addressed but not given a valid command.  If this value is False, the bot
will remain silent."""))

supybot.reply.register('detailedErrors', registry.Boolean(False, """Determines
whether error messages that result from bugs in the bot will show a detailed
error message (the uncaught exception) or a generic error message."""))

supybot.reply.register('errorInPrivate', registry.Boolean(False, """
Determines whether the bot will send error messages to users in private.  You
might want to do this in order to keep channel traffic to minimum.  This can
be used in combination with supybot.reply.errorWithNotice."""))

supybot.reply.register('errorWithNotice', registry.Boolean(False, """
Determines whether the bot will send error messages to users via NOTICE instead
of PRIVMSG.  You might want to do this so users can ignore NOTICEs from the bot
and not have to see error messages; or you might want to use it in combination
with supybot.reply.errorInPrivate so private errors don't open a query window
in most IRC clients."""))

supybot.reply.register('noCapabilityError', registry.Boolean(False, """
Determines whether the bot will send an error message to users who attempt to
call a command for which they do not have the necessary capability.  You may
wish to make this True if you don't want users to understand the underlying
security system preventing them from running certain commands."""))

supybot.reply.register('withPrivateNotice', registry.Boolean(False, """
Determines whether the bot will reply with a private notice to users rather
than sending a message to a channel.  Private notices are particularly nice
because they don't generally cause IRC clients to open a new query window."""))

supybot.reply.register('withNickPrefix', registry.Boolean(True, """
Determines whether the bot will always prefix the user's nick to its reply to
that user's command."""))

supybot.reply.register('whenAddressedByNick', registry.Boolean(True, """
Determines whether the bot will reply when people address it by its nick,
rather than with a prefix character."""))

supybot.reply.register('whenNotAddressed', registry.Boolean(False, """
Determines whether the bot should attempt to reply to all messages even if they
don't address it (either via its nick or a prefix character).  If you set this
to True, you almost certainly want to set supybot.reply.whenNotCommand to
False."""))

# XXX: Removed requireRegistration: it wasn't being used.

supybot.reply.register('requireChannelCommandsToBeSentInChannel',
registry.Boolean(False, """Determines whether the bot will allow you to send
channel-related commands outside of that channel.  Sometimes people find it
confusing if a channel-related command (like Filter.outfilter) changes the
behavior of the channel but was sent outside the channel itself."""))

supybot.register('followIdentificationThroughNickChanges',
registry.Boolean(False, """Determines whether the bot will unidentify someone
when that person changes his or her nick.  Setting this to True will cause the
bot to track such changes.  It defaults to false for a little greater security.
"""))

supybot.register('alwaysJoinOnInvite', registry.Boolean(False, """Determines
whether the bot will always join a channel when it's invited.  If this value
is False, the bot will only join a channel if the user inviting it has the
'admin' capability (or if it's explicitly told to join the channel using the
Admin.join command)"""))

supybot.register('showSimpleSyntax', registry.Boolean(False, """Supybot
normally replies with the full help whenever a user misuses a command.  If this
value is set to True, the bot will only reply with the syntax of the command
(the first line of the docstring) rather than the full help."""))

###
# Replies
###
supybot.register('replies')

registerChannelValue(supybot.replies, 'success',
    registry.NormalizedString("""The operation succeeded.""", """Determines
    what message the bot replies with when a command succeeded."""))

registerChannelValue(supybot.replies, 'error',
    registry.NormalizedString("""An error has occurred and has been logged.
    Please contact this bot's administrator for more information.""", """
    Determines what error message the bot gives when it wants to be
    ambiguous."""))

registerChannelValue(supybot.replies, 'incorrectAuthentication',
    registry.NormalizedString("""Your hostmask doesn't match or your password
    is wrong.""", """Determines what message the bot replies with when someone
    tries to use a command that requires being identified or having a password
    and neither credential is correct."""))

registerChannelValue(supybot.replies, 'noUser',
    registry.NormalizedString("""I can't find that user in my user
    database.  If you didn't give a user name, then I might not know what your
    user is, and you'll need to identify before this command might work.""",
    """Determines what error message the bot replies with when someone tries
    to accessing some information on a user the bot doesn't know about."""))

registerChannelValue(supybot.replies, 'notRegistered',
    registry.NormalizedString("""You must be registered to use this command.
    If you are already registered, you must either identify (using the identify
    command) or add a hostmask matching your current hostmask (using the
    addhostmask command).""", """Determines what error message the bot replies
    with when someone tries to do something that requires them to be registered
    but they're not currently recognized."""))

registerChannelValue(supybot.replies, 'noCapability',
    registry.NormalizedString("""You don't have the %r capability.  If you
    think that you should have this capability, be sure that you are identified
    before trying again.  The 'whoami' command can tell you if you're
    identified.""", """Determines what error message is given when the bot is
    telling someone they aren't cool enough to use the command they tried to
    use."""))

registerChannelValue(supybot.replies, 'genericNoCapability',
    registry.NormalizedString("""You're missing some capability you need.
    This could be because you actually possess the anti-capability for the
    capability that's required of you, or because the channel provides that
    anti-capability by default, or because the global capabilities include
    that anti-capability.  Or, it could be because the channel or the global
    defaultAllow is set to False, meaning that no commands are allowed unless
    explicitly in your capabilities.  Either way, you can't do what you want
    to do.""", """Dertermines what generic error message is given when the bot
    is telling someone that they aren't cool enough to use the command they
    tried to use, and the author of the code calling errorNoCapability didn't
    provide an explicit capability for whatever reason."""))

registerChannelValue(supybot.replies, 'requiresPrivacy',
    registry.NormalizedString("""That operation cannot be done in a
    channel.""", """Determines what error messages the bot sends to people who
    try to do things in a channel that really should be done in private."""))

registerChannelValue(supybot.replies, 'possibleBug',
    registry.NormalizedString("""This may
    be a bug.  If you think it is, please file a bug report at
    <http://sourceforge.net/tracker/?func=add&group_id=58965&atid=489447>.""",
    """Determines what message the bot sends when it thinks you've encountered
    a bug that the developers don't know about."""))
###
# End supybot.replies.
###

supybot.register('maxHistoryLength', registry.Integer(1000, """Determines
how many old messages the bot will keep around in its history.  Changing this
variable will not take effect until the bot is restarted."""))

supybot.register('nickmods', registry.CommaSeparatedListOfStrings(
    '__%s__,%s^,%s`,%s_,%s__,_%s,__%s,[%s]'.split(','),
    """A list of modifications to be made to a nick when the nick the bot tries
    to get from the server is in use.  There should be one %s in each string;
    this will get replaced with the original nick."""))

supybot.register('throttleTime', registry.Float(1.0, """A floating point
number of seconds to throttle queued messages -- that is, messages will not
be sent faster than once per throttleTime seconds."""))

supybot.register('snarfThrottle', registry.Float(10.0, """A floating point
number of seconds to throttle snarfed URLs, in order to prevent loops between
two bots snarfing the same URLs and having the snarfed URL in the output of
the snarf message."""))

supybot.register('threadAllCommands', registry.Boolean(False, """Determines
whether the bot will automatically thread all commands.  At this point this
option exists almost exclusively for debugging purposes; it can do very little
except to take up more CPU."""))

supybot.register('pingServer', registry.Boolean(True, """Determines whether
the bot will send PINGs to the server it's connected to in order to keep the
connection alive and discover earlier when it breaks.  Really, this option
only exists for debugging purposes: you always should make it True unless
you're testing some strange server issues."""))

supybot.register('pingInterval', registry.Integer(120, """Determines the
number of seconds between sending pings to the server, if pings are being sent
to the server."""))

supybot.register('upkeepInterval', registry.PositiveInteger(3600, """Determines
the number of seconds between running the upkeep function that flushes
(commits) open databases, collects garbage, and records some useful statistics
at the debugging level."""))

supybot.register('flush', registry.Boolean(True, """Determines whether the bot
will periodically flush data and configuration files to disk.  Generally, the
only time you'll want to set this to False is when you want to modify those
configuration files by hand and don't want the bot to flush its current version
over your modifications.  Do note that if you change this to False inside the
bot, your changes won't be flushed.  To make this change permanent, you must
edit the registry yourself."""))

supybot.register('httpPeekSize', registry.PositiveInteger(4096, """Determines
how many bytes the bot will 'peek' at when looking through a URL for a
doctype or title or something similar.  It'll give up after it reads this many
bytes."""))

class SocketTimeout(registry.PositiveInteger):
    def setValue(self, v):
        registry.PositiveInteger.setValue(self, v)
        socket.setdefaulttimeout(self.value)
        
supybot.register('defaultSocketTimeout', SocketTimeout(10, """Determines what
the default timeout for socket objects will be.  This means that *all* sockets
will timeout when this many seconds has gone by (unless otherwise modified by
the author of the code that uses the sockets)."""))

###
# Driver stuff.
###
supybot.register('drivers')
supybot.drivers.register('poll', registry.Float(1.0, """Determines the default
length of time a driver should block waiting for input."""))

class ValidDriverModule(registry.OnlySomeStrings):
    validStrings = ('socketDrivers', 'twistedDrivers', 'asyncoreDrivers')

supybot.drivers.register('module', ValidDriverModule('socketDrivers', """
Determines what driver module the bot will use.  socketDrivers, a simple
driver based on timeout sockets, is used by default because it's simple and
stable.  asyncoreDrivers is a bit older (and less well-maintained) but allows
you to integrate with asyncore-based applications.  twistedDrivers is very
stable and simple, and if you've got Twisted installed, is probably your best
bet."""))

supybot.register('directories')
supybot.directories.register('conf', registry.String('conf', """
Determines what directory configuration data is put into."""))
supybot.directories.register('data', registry.String('data', """
Determines what directory data is put into."""))
supybot.directories.register('plugins',
registry.CommaSeparatedListOfStrings([_srcDir,_pluginsDir],
"""Determines what directories the bot will look for plugins in.  Accepts a
comma-separated list of strings.  This means that to add another directory,
you can nest the former value and add a new one.  E.g. you can say: bot:
'config supybot.directories.plugins [config supybot.directories.plugins],
newPluginDirectory'."""))

supybot.register('databases')
supybot.databases.register('users')
supybot.databases.users.register('filename', registry.String('users.conf', """
Determines what filename will be used for the users database.  This file will
go into the directory specified by the supybot.directories.conf
variable."""))
supybot.databases.users.register('timeoutIdentification',
registry.Integer(0, """Determines how long it takes identification to time
out.  If the value is less than or equal to zero, identification never
times out."""))
supybot.databases.users.register('hash', registry.Boolean(False, """
Determines whether the passwords in the user database will be hashed by
default."""))

supybot.databases.register('ignores')
supybot.databases.ignores.register('filename', registry.String('ignores.conf',
"""Determines what filename will be used for the ignores database.  This file
will go into the directory specified by the supybot.directories.conf
variable."""))

supybot.databases.register('channels')
supybot.databases.channels.register('filename',registry.String('channels.conf',
"""Determines what filename will be used for the channels database.  This file
will go into the directory specified by the supybot.directories.conf
variable."""))

supybot.register('plugins') # This will be used by plugins, but not here.

###############################
###############################
###############################
# DO NOT EDIT PAST THIS POINT #
###############################
###############################
###############################
version ='0.77.2'

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Commands specific to OSU;
(The Ohio State University, <http://www.ohio-state.edu/>)
"""

__revision__ = "$Id: OSU.py,v 1.16 2004/01/29 23:58:28 jamessan Exp $"

import plugins

import urllib2

import utils
import privmsgs
import callbacks


buildings = {
    'AA': 'Agricultural Administration Building, ' \
          '2120 Fyffe Road, Columbus, Ohio, 43210',
    'AC': 'ATI Student Activities Center, ' \
          'Agriculture Tech Inst, Wooster, Ohio, 44691',
    'AE': 'Agricultural Engineering, ' \
          '590 Woody Hayes Drive, Columbus, Ohio, 43210',
    'AF': 'Wagner, ATI Fairgrounds, Wooster, Ohio, 44691',
    'AG': 'ATI Greenhouse 3, Agriculture Tech Inst, Wooster, Ohio, 44691',
    'AK': 'Applecreek, Agriculture Tech Inst, Wooster, Ohio, 44691',
    'AM': 'Allied Medical Professions Building, ' \
          '1583 Perry Street, Columbus, Ohio, 43210',
    'AO': 'Airport Operations Building, ' \
          '2160 W Case Road, Columbus, Ohio, 43235',
    'AP': 'Arps Hall, 1945 N High Street, Columbus, Ohio, 43210',
    'AR': 'ATI Residence Hall, Agriculture Tech Inst, Wooster, Ohio, 44691',
    'AS': 'Animal Science Building, 2029 Fyffe Road, Columbus, Ohio, 43210',
    'AT': 'Halterman Hall (ATI), 1328 Dover Road, Wooster, Ohio, 44691',
    'AV': 'Aviation Building, 164 W 19th Avenue, Columbus, Ohio, 43210',
    'BA': 'Browning Amphitheatre, Mirror Lake, Columbus, Ohio, 43210',
    'BE': 'Baker Systems Engineering, 1971 Neil Avenue, Columbus, Ohio, 43210',
    'BF': 'Bromfield Hall, 1660 University Drive, Mansfield, Ohio, 44906',
    'BH': 'Bevis Hall, 1080 Carmack Road, Columbus, Ohio, 43210',
    'BI': 'Biological Sciences Building, ' \
          '484 W 12th Avenue, Columbus, Ohio, 43210',
    'BK': 'Bricker Hall, 190 N Oval Mall, Columbus, Ohio, 43210',
    'BL': 'Boyd Laboratory, 155 W Woodruff Avenue, Columbus, Ohio, 43210',
    'BO': 'Bolz Hall, 2036 Neil Avenue Mall, Columbus, Ohio, 43210',
    'BR': 'Brown Hall, 190 W 17th Avenue, Columbus, Ohio, 43210',
    'BZ': 'Botany & Zoology Building, 1735 Neil Avenue, Columbus, Ohio, 43210',
    'CC': 'Central Classroom Building, ' \
          '2009 Millikin Road, Columbus, Ohio, 43210',
    'CE': 'Celeste Laboratory Of Chemistry, ' \
          '120 W 18th Avenue, Columbus, Ohio, 43210',
    'CH': 'Cockins Hall, 1958 Neil Avenue, Columbus, Ohio, 43210',
    'CK': 'Cook Hall, 4240 Campus Drive, Lima, Ohio, 45804',
    'CL': 'Caldwell Laboratory, 2024 Neil Avenue, Columbus, Ohio, 43210',
    'CM': 'Campbell Hall, 1787 Neil Avenue, Columbus, Ohio, 43210',
    'CT': 'Fawcett Center For Tomorrow, ' \
          '2400 Olentangy River, Columbus, Ohio, 43210',
    'CV': 'Converse Hall, 2121 Tuttle Park Pl, Columbus, Ohio, 43210',
    'CX': 'Community Extension Center, ' \
          '905 Mt Vernon Avenue, Columbus, Ohio, 43203',
    'CZ': 'Cunz Hall, 1841 Millikin Road, Columbus, Ohio, 43210',
    'DB': 'Derby Hall, 154 N Oval Mall, Columbus, Ohio, 43210',
    'DE': 'Denney Hall, 164 W 17th Avenue, Columbus, Ohio, 43210',
    'DI': 'Drinko Hall, 55 W 12th Avenue, Columbus, Ohio, 43210',
    'DK': 'Dakan Hall, 674 W Lane Avenue, Columbus, Ohio, 43210',
    'DL': 'Dreese Laboratories, 2015 Neil Avenue, Columbus, Ohio, 43210',
    'DN': 'Doan Hall, 410 W 10th Avenue, Columbus, Ohio, 43210',
    'DO': 'Dodd Hall, 480 W 9th Avenue, Columbus, Ohio, 43210',
    'DR': 'Drake Union, 1849 Cannon Drive, Columbus, Ohio, 43210',
    'DU': 'Dulles Hall, 230 W 17th Avenue, Columbus, Ohio, 43210',
    'DV': 'Davis Medical Research Center, ' \
          '480 W 9th Avenue, Columbus, Ohio, 43210',
    'EA': '209 W Eighteenth Building, ' \
          '209 West 18th Avenue, Columbus, Ohio, 43210',
    'EC': 'Eisenhower Memorial Center, ' \
          '1640 University Drive, Mansfield, Ohio, 44906',
    'EL': 'Evans Laboratory, 88 W 18th Avenue, Columbus, Ohio, 43210',
    'EN': 'Enarson Hall, 154 W 12th Avenue, Columbus, Ohio, 43210',
    'FA': 'Fisher Auditorium, OARDC-Wooster, Wooster, Ohio, 44691',
    'FF': 'French Field House, 460 Woody Hayes Drive, Columbus, Ohio, 43210',
    'FH': 'Founders Hall, 1179 University Drive, Newark, Ohio, 43055',
    'FL': 'Fontana Laboratories, 116 W 19th Avenue, Columbus, Ohio, 43210',
    'FR': 'Fry Hall, 338 W 10th Avenue, Columbus, Ohio, 43210',
    'FT': 'Fallerius Technical Educ Center, ' \
          '2441 Kenwood Circle, Mansfield, Ohio, 44906',
    'GA': 'Galvin Hall, 4240 Campus Drive, Lima, Ohio, 45804',
    'GB': 'General Biology Annex, 1791 Neil Avenue, Columbus, Ohio, 43210',
    'GH': 'Golf Course Club House, 3605 Tremont Road, Columbus, Ohio, 43221',
    'GL': 'Goss Laboratory, 1925 Coffey Road, Columbus, Ohio, 43210',
    'GR': 'Graves Hall, 333 W 10th Avenue, Columbus, Ohio, 43210',
    'HA': 'Hayes Hall, 108 N Oval Mall, Columbus, Ohio, 43210',
    'HC': 'Hopkins Hall, 128 N Oval Mall, Columbus, Ohio, 43210',
    'HG': 'Howlett Greenhouses, 680 Tharp St, Columbus, Ohio, 43210',
    'HH': 'Hagerty Hall, 1775 College Road, Columbus, Ohio, 43210',
    'HI': 'Hitchcock Hall, 2070 Neil Avenue, Columbus, Ohio, 43210',
    'HK': 'Haskett Hall, 156 W 19th Avenue, Columbus, Ohio, 43210',
    'HL': 'Hale Hall, 153 W 12th Avenue, Columbus, Ohio, 43210',
    'HM': 'Hamilton Hall, 1645 Neil Avenue, Columbus, Ohio, 43210',
    'HN': 'Kuhn Honors House, 220 W 12th Avenue, Columbus, Ohio, 43210',
    'HP': 'Hopewell Hall (Newark), 1179 University Drive, Newark, Ohio, 43055',
    'HS': 'Health Science Library, 376 W 10th Avenue, Columbus, Ohio, 43210',
    'HT': 'Howlett Hall, 2001 Fyffe Court, Columbus, Ohio, 43210',
    'HU': 'Hughes Hall, 1899 College Road, Columbus, Ohio, 43210',
    'IH': 'Independence Hall, 1923 Neil Avenue Mall, Columbus, Ohio, 43210',
    'IR': 'Ice Rink, 390 Woody Hayes Drive, Columbus, Ohio, 43210',
    'IV': 'Ives Hall, 2073 Neil Avenue, Columbus, Ohio, 43210',
    'JA': 'James Cancer Hosp & Research Inst, ' \
          '300 W 10th Avenue, Columbus, Ohio, 43210',
    'JR': 'Journalism Building, 242 W 18th Avenue, Columbus, Ohio, 43210',
    'KH': 'Kottman Hall, 2021 Coffey Road, Columbus, Ohio, 43210',
    'KL': 'Koffolt Laboratories, 140 W 19th Avenue, Columbus, Ohio, 43210',
    'KR': '1224 Kinnear Road, 1224 Kinnear Road, Columbus, Ohio, 43212',
    'LC': 'Ohio Legal Center, 33 W 11th Avenue, Columbus, Ohio, 43201',
    'LI': 'Main Library, 1858 Neil Avenue Mall, Columbus, Ohio, 43210',
    'LK': 'Larkins Hall, 337 W 17th Avenue, Columbus, Ohio, 43210',
    'LO': 'Lord Hall, 124 W 17th Avenue, Columbus, Ohio, 43210',
    'LS': 'Reed Student Activities Building, ' \
          '4240 Campus Drive, Lima, Ohio, 45804',
    'LT': 'Lincoln Tower, 1800 Cannon Drive, Columbus, Ohio, 43210',
    'LZ': 'Lazenby Hall, 1827 Neil Avenue Mall, Columbus, Ohio, 43210',
    'MA': 'Mathematics Building, 231 W 18th Avenue, Columbus, Ohio, 43210',
    'MC': 'McCampbell Hall, 1581 Dodd Drive, Columbus, Ohio, 43210',
    'ME': 'Meiling Hall, 370 W 9th Avenue, Columbus, Ohio, 43210',
    'ML': 'Mendenhall Laboratory, 125 S Oval Mall, Columbus, Ohio, 43210',
    'MM': 'Mershon Auditorium, 1871 N High St, Columbus, Ohio, 43210',
    'MN': '1501 Neil Avenue, 1501 Neil Avenue, Columbus, Ohio, 43201',
    'MO': 'Mount Hall, 1050 Carmack Road, Columbus, Ohio, 43210',
    'MP': 'McPherson Chemical Laboratory, ' \
          '140 W 18th Avenue, Columbus, Ohio, 43210',
    'MQ': 'MacQuigg Laboratory, 105 W Woodruff Avenue, Columbus, Ohio, 43210',
    'MR': 'Morrill Hall (Marion), 1465 Mt Vernon Avenue, Marion, Ohio, 43302',
    'MS': 'Means Hall, 1654 Upham Drive, Columbus, Ohio, 43210',
    'MT': 'Morrill Tower, 1900 Cannon Drive, Columbus, Ohio, 43210',
    'NE': 'Neil-17th Building, 1949 Neil Avenue, Columbus, Ohio, 43210',
    'NH': 'Newton Hall, 1585 Neil Avenue, Columbus, Ohio, 43210',
    'NL': 'Neil Hall, 1634 Neil Avenue, Columbus, Ohio, 43210',
    'NR': 'Jesse Owens Recreation Center North, ' \
          '2151 Neil Avenue, Columbus, Ohio, 43210',
    'OR': 'Orton Hall, 155 S Oval Mall, Columbus, Ohio, 43210',
    'OU': 'Ohio Union, 1739 N High St, Columbus, Ohio, 43210',
    'OV': 'Ovalwood Hall (Mansfield), ' \
          '1680 University Drive, Mansfield, Ohio, 44906',
    'OX': 'Oxley Hall, 1712 Neil Avenue, Columbus, Ohio, 43210',
    'PA': 'Page Hall, 1810 College Road, Columbus, Ohio, 43210',
    'PH': 'Postle Hall, 305 W 12th Avenue, Columbus, Ohio, 43210',
    'PK': 'Parks Hall, 500 W 12th Avenue, Columbus, Ohio, 43210',
    'PL': 'Plumb Hall, 2027 Coffey Road, Columbus, Ohio, 43210',
    'PN': '1478 Pennsylvania Avenue, ' \
          '1478 Pennsylvania Avenue, Columbus, Ohio, 43201',
    'PO': 'Pomerene Hall, 1760 Neil Avenue, Columbus, Ohio, 43210',
    'PR': 'Pressey Hall, 1070 Carmack Road, Columbus, Ohio, 43210',
    'RA': 'Ramseyer Hall, 29 W Woodruff Avenue, Columbus, Ohio, 43210',
    'RC': 'Research Center, 1314 Kinnear Road, Columbus, Ohio, 43212',
    'RD': 'Rhodes Hall, 450 W 10th Avenue, Columbus, Ohio, 43210',
    'RF': 'Riffe Building, 496 W 12th Avenue, Columbus, Ohio, 43210',
    'RH': 'Rightmire Hall, 1060 Carmack Road, Columbus, Ohio, 43210',
    'RL': 'Robinson Laboratory, 206 W 18th Avenue, Columbus, Ohio, 43210',
    'RY': 'Royer Student Activities Center, ' \
          '85 Curl Drive, Columbus, Ohio, 43210',
    'SA': 'Foundry Metals & Glass Building, ' \
          '1055 Carmack Road, Columbus, Ohio, 43210',
    'SC': 'Scott Hall, 1090 Carmack Road, Columbus, Ohio, 43210',
    'SD': 'Alber Student Center, 1465 Mt Vernon Avenue, Marion, Ohio, 43302',
    'SE': 'Steeb Hall, Se 70 W 11th Avenue, Columbus, Ohio, 43210',
    'SH': 'Stillman Hall, 1947 College Road, Columbus, Ohio, 43210',
    'SI': 'Sisson Hall, 1900 Coffey Road, Columbus, Ohio, 43210',
    'SJ': 'St John Arena, 410 Woody Hayes Drive, Columbus, Ohio, 43210',
    'SK': 'Skou Hall, Agriculture Tech Inst, Wooster, Ohio, 44691',
    'SL': 'Starling Loving Hall A, 320 W 10th Avenue, Columbus, Ohio, 43210',
    'SM': 'Smith Laboratory, 174 W 18th Avenue, Columbus, Ohio, 43210',
    'SN': 'Gibraltar Stonlab, Gibraltar Island, Put-in-bay, Ohio, 43456',
    'SP': 'ATI Shop, Agriculture Tech Inst, Wooster, Ohio, 44691',
    'SR': 'Jesse Owens Recreation Center South, ' \
          '175 W 11th Avenue, 175 West 11th Avenue, Ohio, OH',
    'ST': 'Ohio Stadium, 411 Woody Hayes Drive, Columbus, Ohio, 43210',
    'SU': 'Sullivant Hall, 1813 N High St, Columbus, Ohio, 43210',
    'TE': 'Marion Technical Education Center, ' \
          '1467 Mt Vernon Avenue, Marion, Ohio, 43302',
    'TL': 'Technical Education Building (Lima), ' \
          '4240 Campus Drive, Lima, Ohio, 45804',
    'TO': 'Townshend Hall, 1885 Neil Avenue Mall, Columbus, Ohio, 43210',
    'TT': 'Taylor Tower, 50 Curl Drive, Columbus, Ohio, 43210',
    'UH': 'University Hall, 230 N Oval Mall, Columbus, Ohio, 43210',
    'UP': 'Upham Hall, 473 W 12th Avenue, Columbus, Ohio, 43210',
    'VE': 'Veterinary Hospital, 601 Vernon Tharp St, Columbus, Ohio, 43210',
    'VG': 'Van De Graaff Laboratory, 1302 Kinnear Road, Columbus, Ohio, 43212',
    'VH': 'Vivian Hall, 2121 Fyffe Road, Columbus, Ohio, 43210',
    'WA': 'Watts Hall, 2041 College Road, Columbus, Ohio, 43210',
    'WE': 'Welding Engineering Laboratory, ' \
          '190 W 19th Avenue, Columbus, Ohio, 43210',
    'WG': 'Weigel Hall, 1866 College Road, Columbus, Ohio, 43210',
    'WI': 'Wiseman Hall, 400 W 12th Avenue, Columbus, Ohio, 43210',
    'WL': 'Williams, OARDC-Wooster, Wooster, Ohio, 44691',
    'WO': 'Womens Field House, 1801 Neil Avenue, Columbus, Ohio, 43210',
    'WR': 'Jesse Owens Recreation Center West, ' \
          '1031 Carmack Road, Columbus, Ohio, 43210',
    'WS': 'Wilce Health Center, 1875 Millikin Road, Columbus, Ohio, 43210',
    'WX': 'Wexner Center, 1850 College Road, Columbus, Ohio, 43210'
}

class OSU(callbacks.Privmsg):
    def email(self, irc, msg, args):
        """<first name> <middle initial> <last name>

        Returns possible email address matches for the given name.
        """
        s = '.'.join(args)
        url = 'http://www.ohio-state.edu/cgi-bin/inquiry2.cgi?keyword=%s' % s
        try:
            fd = urllib2.urlopen(url)
            data = fd.read()
            emails = []
            for line in data.splitlines():
                line.strip()
                if 'Published address' in line:
                    emails.append(line.split()[-1])
            if len(emails) == 0:
                irc.reply('There seem to be no matches to that name.')
            elif len(emails) == 1:
                irc.reply(emails[0])
            else:
                irc.reply('Possible matches: %s' % ', '.join(emails))
        except Exception, e:
            irc.error(utils.exnToString(e))
    email = privmsgs.thread(email)

    def building(self, irc, msg, args):
        """<building abbreviation>

        Returns the address and full name of an OSU building based on its
        standard two-letter abbreviation.
        """
        building = privmsgs.getArgs(args)
        try:
            irc.reply(buildings[building.upper()])
        except KeyError:
            irc.reply('I don\'t know of any such OSU building.')


Class = OSU

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

__revision__ = "$Id: twistedDrivers.py,v 1.25 2004/02/16 08:10:50 jemfinch Exp $"

import fix

import time

import log
import conf
import ircdb
import drivers
import ircmsgs

from twisted.internet import reactor
from twisted.protocols.basic import LineReceiver
from twisted.internet.protocol import ReconnectingClientFactory

class TwistedRunnerDriver(drivers.IrcDriver):
    def run(self):
        try:
            reactor.iterate(conf.supybot.drivers.poll())
        except:
            log.exception('Uncaught exception outside reactor:')

class SupyIrcProtocol(LineReceiver):
    delimiter = '\n'
    MAX_LENGTH = 1024
    def __init__(self):
        self.mostRecentCall = reactor.callLater(1, self.checkIrcForMsgs)

    def lineReceived(self, line):
        start = time.time()
        msg = ircmsgs.IrcMsg(line)
        #log.debug('Time to parse IrcMsg: %s', time.time()-start)
        self.irc.feedMsg(msg)

    def checkIrcForMsgs(self):
        if self.connected:
            msg = self.irc.takeMsg()
            if msg:
                self.transport.write(str(msg))
        self.mostRecentCall = reactor.callLater(1, self.checkIrcForMsgs)

    def connectionLost(self, failure):
        self.mostRecentCall.cancel()
        self.irc.reset()
        log.warning(failure.getErrorMessage())

    def connectionMade(self):
        self.irc.reset()
        self.irc.driver = self

    def die(self):
        log.info('Driver for %s dying.', self.irc)
        self.factory.continueTrying = False
        self.transport.loseConnection()

    def reconnect(self):
        self.transport.loseConnection()


class SupyReconnectingFactory(ReconnectingClientFactory):
    maxDelay = 300
    protocol = SupyIrcProtocol
    def __init__(self, (server, port), irc):
        self.irc = irc
        self.server = (server, port)
        reactor.connectTCP(server, port, self)

    def buildProtocol(self, addr):
        protocol = ReconnectingClientFactory.buildProtocol(self, addr)
        protocol.irc = self.irc
        return protocol
    
    def die(self):
        pass

Driver = SupyReconnectingFactory

try:
    ignore(poller)
except NameError:
    poller = TwistedRunnerDriver()
# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Includes various accessories for callbacks.Privmsg based callbacks.
"""

__revision__ = "$Id: privmsgs.py,v 1.81 2004/02/24 21:16:53 jemfinch Exp $"

import fix

import time
import types
import threading

import conf
import utils
import world
import ircdb
import ircmsgs
import ircutils
import callbacks
import structures

def getChannel(msg, args, raiseError=True):
    """Returns the channel the msg came over or the channel given in args.

    If the channel was given in args, args is modified (the channel is
    removed).
    """
    if args and ircutils.isChannel(args[0]):
        if conf.supybot.reply.requireChannelCommandsToBeSentInChannel():
            if args[0] != msg.args[0]:
                s = 'Channel commands must be sent in the channel to which ' \
                    'they apply; if this is not the behavior you desire, ' \
                    'ask the bot\'s administrator to change the registry ' \
                    'variable ' \
                    'supybot.reply.requireChannelCommandsToBeSentInChannel ' \
                    'to False.'
                raise callbacks.Error, s
        return args.pop(0)
    elif ircutils.isChannel(msg.args[0]):
        return msg.args[0]
    else:
        if raiseError:
            raise callbacks.Error, 'Command must be sent in a channel or ' \
                                   'include a channel in its arguments.'
        else:
            return None

def getArgs(args, required=1, optional=0):
    """Take the required/optional arguments from args.

    Always returns a list of size required + optional, filling it with however
    many empty strings is necessary to fill the tuple to the right size.  If
    there is only one argument, a string containing that argument is returned.

    If there aren't enough args even to satisfy required, raise an error and
    let the caller handle sending the help message.
    """
    assert not isinstance(args, str), 'args should be a list.'
    assert not isinstance(args, ircmsgs.IrcMsg), 'args should be a list.'
    if len(args) < required:
        raise callbacks.ArgumentError
    if len(args) < required + optional:
        ret = list(args) + ([''] * (required + optional - len(args)))
    elif len(args) >= required + optional:
        ret = list(args[:required + optional - 1])
        ret.append(' '.join(args[required + optional - 1:]))
    if len(ret) == 1:
        return ret[0]
    else:
        return ret

def checkCapability(f, capability):
    """Makes sure a user has a certain capability before a command will run."""
    def newf(self, irc, msg, args):
        if ircdb.checkCapability(msg.prefix, capability):
            f(self, irc, msg, args)
        else:
            self.log.warning('%r attempted %s without %s.',
                             msg.prefix, f.func_name, capability)
            irc.errorNoCapability(capability)
    return utils.changeFunctionName(newf, f.func_name, f.__doc__)

def checkChannelCapability(f, capability):
    """Makes sure a user has a certain channel capability before running f.

    Do note that you need to add a "channel" argument to your argument list.
    """
    def newf(self, irc, msg, args, *L):
        channel = getChannel(msg, args)
        chancap = ircdb.makeChannelCapability(channel, capability)
        if ircdb.checkCapability(msg.prefix, chancap):
            L += (channel,)
            ff = types.MethodType(f, self, self.__class__)
            ff(irc, msg, args, *L)
        else:
            self.log.warning('%r attempted %s without %s.',
                             msg.prefix, f.func_name, capability)
            irc.errorNoCapability(chancap)
    return utils.changeFunctionName(newf, f.func_name, f.__doc__)

def _threadedWrapMethod(f):
    """A function to wrap methods that are to be run in a thread, so that the
    callback's threaded attribute is set to True while the thread is running.
    """
    def newf(self, *args, **kwargs):
        originalThreaded = self.threaded
        try:
            self.threaded = True
            return f(self, *args, **kwargs)
        finally:
            self.threaded = originalThreaded
    return utils.changeFunctionName(newf, f.func_name, f.__doc__)
            
def thread(f):
    """Makes sure a command spawns a thread when called."""
    f = _threadedWrapMethod(f)
    def newf(self, irc, msg, args, *L):
        ff = types.MethodType(f, self, self.__class__)
        t = callbacks.CommandThread(target=irc._callCommand,
                                    args=(f.func_name, ff, self))
        t.start()
    return utils.changeFunctionName(newf, f.func_name, f.__doc__)

def channel(f):
    """Gives the command an extra channel arg as if it had called getChannel"""
    def newf(self, irc, msg, args, *L):
        channel = getChannel(msg, args)
        L = (channel,) + L
        ff = types.MethodType(f, self, self.__class__)
        ff(irc, msg, args, *L)
    return utils.changeFunctionName(newf, f.func_name, f.__doc__)

def urlSnarfer(f):
    """Protects the snarfer from loops and whatnot."""
    f = _threadedWrapMethod(f)
    q = structures.smallqueue()
    def newf(self, irc, msg, match, *L):
        channel = msg.args[0]
        if ircutils.isChannel(channel):
            c = ircdb.channels.getChannel(channel)
            if c.lobotomized:
                self.log.info('Refusing to snarf in %s: lobotomized.', channel)
                return
        now = time.time()
        cutoff = now - conf.supybot.snarfThrottle()
        while q and q[0][2] < cutoff:
            q.dequeue()
        url = match.group(0)
        for (qUrl, target, when) in q:
            if url == qUrl and target == channel and not world.testing:
                self.log.warning('Not snarfing %s from %r.', url, msg.prefix)
                return
        else:
            q.enqueue((url, channel, now))
            if self.threaded:
                f(self, irc, msg, match, *L)
            else:
                L = list(L)
                world.threadsSpawned += 1
                t = threading.Thread(target=f, args=[self,irc,msg,match]+L)
                t.setDaemon(True)
                t.start()
    newf = utils.changeFunctionName(newf, f.func_name, f.__doc__)
    newf.q = q # This isn't necessary, it's just for debugging.
    return newf
        

class CapabilityCheckingPrivmsg(callbacks.Privmsg):
    """A small subclass of callbacks.Privmsg that checks self.capability
    before allowing any command to be called.
    """
    capability = '' # To satisfy PyChecker
    def callCommand(self, f, irc, msg, args):
        if ircdb.checkCapability(msg.prefix, self.capability):
            callbacks.Privmsg.callCommand(self, f, irc, msg, args)
        else:
            self.log.warning('%r tried to call %s without %s.',
                             msg.prefix, f.im_func.func_name, self.capability)
            irc.errorNoCapability(self.capability)


# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Handles configuration of the bot while it's running.
"""

__revision__ = "$Id: Config.py,v 1.20 2004/04/18 02:40:18 jemfinch Exp $"

import getopt

import conf
import utils
import world
import ircdb
import plugins
import ircutils
import privmsgs
import registry
import callbacks

###
# Now, to setup the registry.
###

class InvalidRegistryName(callbacks.Error):
    pass

def getWrapper(name):
    parts = name.split('.')
    if not parts or parts[0] != 'supybot':
        raise InvalidRegistryName, name
    group = conf.supybot
    parts.pop(0)
    while parts:
        try:
            group = group.get(parts.pop(0))
        except registry.NonExistentRegistryEntry:
            raise InvalidRegistryName, name
    return group

def getCapability(name):
    capability = 'owner' # Default to requiring the owner capability.
    parts = name.split('.')
    while parts:
        part = parts.pop()
        if ircutils.isChannel(part):
            # If a registry value has a channel in it, it requires a channel.op
            # capability, or so we assume.  We'll see if we're proven wrong.
            capability = ircdb.makeChannelCapability(part, 'op')
        ### Do more later, for specific capabilities/sections.
    return capability


class Config(callbacks.Privmsg):
    def callCommand(self, method, irc, msg, *L):
        try:
            callbacks.Privmsg.callCommand(self, method, irc, msg, *L)
        except InvalidRegistryName, e:
            irc.error('%r is not a valid configuration variable.' % e.args[0])
        except registry.InvalidRegistryValue, e:
            irc.error(str(e))

    def _canonicalizeName(self, name):
        if not name.startswith('supybot'):
            name = 'supybot.' + name
        return name

    def list(self, irc, msg, args):
        """[--groups] <group>

        Returns the configuration variables available under the given
        configuration <group>.  If --groups is given, return the subgroups of
        the <group>.
        """
        (optlist, rest) = getopt.getopt(args, '', ['groups'])
        groups = False
        for (name, arg) in optlist:
            if name == '--groups':
                groups = True
        name = privmsgs.getArgs(rest)
        name = self._canonicalizeName(name)
        group = getWrapper(name)
        if groups:
            L = []
            for (vname, v) in group.children.iteritems():
                if v.added:
                    L.append(vname)
            if L:
                utils.sortBy(str.lower, L)
                irc.reply(utils.commaAndify(L))
            else:
                irc.reply('%s has no subgroups.' % name)
        else:
            try:
                L = zip(*group.getValues(fullNames=False))[0]
                irc.reply(utils.commaAndify(L))
            except TypeError:
                irc.error('There don\'t seem to be any values in %s'%name)

    def search(self, irc, msg, args):
        """<word>

        Searches for <word> in the current configuration variables.
        """
        word = privmsgs.getArgs(args)
        word = word.lower()
        L = []
        for (name, _) in conf.supybot.getValues(getChildren=True):
            if word in name.lower():
                L.append(name)
        if L:
            irc.reply(utils.commaAndify(L))
        else:
            irc.reply('There were no matching configuration variables.')

    def config(self, irc, msg, args):
        """<name> [<value>]

        If <value> is given, sets the value of <name> to <value>.  Otherwise,
        returns the current value of <name>.  You may omit the leading
        "supybot." in the name if you so choose.
        """
        if len(args) >= 2:
            self._set(irc, msg, args)
        else:
            self._get(irc, msg, args)

    def _get(self, irc, msg, args):
        """<name>

        Shows the current value of the configuration variable <name>.
        """
        name = privmsgs.getArgs(args)
        name = self._canonicalizeName(name)
        wrapper = getWrapper(name)
        if hasattr(wrapper, 'value'):
            if not wrapper.private:
                irc.reply(str(wrapper))
            else:
                capability = getCapability(name)
                if ircdb.checkCapability(msg.prefix, capability):
                    irc.reply(str(wrapper))
                else:
                    irc.errorNoCapability(capability)
        else:
            irc.error('That registry variable has no value.  Use the list '
                      'command in this plugin to see what values are '
                      'available in this group.')

    def _set(self, irc, msg, args):
        """<name> <value>

        Sets the current value of the configuration variable <name> to <value>.
        """
        (name, value) = privmsgs.getArgs(args, required=2)
        name = self._canonicalizeName(name)
        capability = getCapability(name)
        if ircdb.checkCapability(msg.prefix, capability):
            wrapper = getWrapper(name)
            wrapper.set(value)
            irc.replySuccess()
        else:
            irc.errorNoCapability(capability)

    def help(self, irc, msg, args):
        """<name>

        Returns the description of the configuration variable <name>.
        """
        name = privmsgs.getArgs(args)
        name = self._canonicalizeName(name)
        wrapper = getWrapper(name)
        if hasattr(wrapper, 'help'):
            irc.reply(wrapper.help)
        else:
            irc.error('%s has no help.' % name)

    def default(self, irc, msg, args):
        """<name>

        Returns the default value of the configuration variable <name>.
        """
        name = privmsgs.getArgs(args)
        name = self._canonicalizeName(name)
        wrapper = getWrapper(name)
        v = wrapper.__class__(wrapper.default, '')
        irc.reply(str(v))

    def reload(self, irc, msg, args):
        """takes no arguments

        Reloads the various configuration files (user database, channel
        database, registry, etc.).
        """
        ircdb.users.reload()
        ircdb.channels.reload()
        registry.open(world.registryFilename)
        irc.replySuccess()
    reload = privmsgs.checkCapability(reload, 'owner')
        


Class = Config

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2003, Daniel DiPaolo
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Moobot factoid compatibility module.  Overrides the replyWhenNotCommand
behavior so that when someone addresses the bot with anything other than a
command, it checks the factoid database for a key that matches what was said
and if nothing is found, responds with an entry from the "dunno" database.
"""

__revision__="$Id: MoobotFactoids.py,v 1.62 2004/03/11 04:50:09 ddipaolo Exp $"

import plugins

import os
import sets
import time
import shlex
import string
import random
from itertools import imap
from cStringIO import StringIO

import registry

import conf
import ircdb
import utils
import ircmsgs
import ircutils
import privmsgs
import callbacks

import Owner

try:
    import sqlite
except ImportError:
    raise callbacks.Error, 'You need to have PySQLite installed to use this ' \
                           'plugin.  Download it at <http://pysqlite.sf.net/>'

dbfilename = os.path.join(conf.supybot.directories.data(), 'MoobotFactoids')


allchars = string.maketrans('', '')
class OptionList(object):
    validChars = allchars.translate(allchars, '|()')
    def _insideParens(self, lexer):
        ret = []
        while True:
            token = lexer.get_token()
            if not token:
                return '(%s' % ''.join(ret) #)
            elif token == ')':
                if len(ret) > 1:
                    if '|' in ret:
                        L = map(''.join,
                                utils.itersplit('|'.__eq__, ret,
                                                yieldEmpty=True))
                        return random.choice(L)
                    else:
                        return ''.join(ret)
                    return [x for x in ret if x != '|']
                elif len(ret) == 1:
                        return '(%s)' % ret[0]
                else:
                    return '()'
            elif token == '(':
                ret.append(self._insideParens(lexer))
            elif token == '|':
                ret.append(token)
            else:
                ret.append(token)

    def tokenize(self, s):
        lexer = shlex.shlex(StringIO(s))
        lexer.commenters = ''
        lexer.quotes = ''
        lexer.whitespace = ''
        lexer.wordchars = self.validChars
        ret = []
        while True:
            token = lexer.get_token()
            if not token:
                break
            elif token == '(':
                ret.append(self._insideParens(lexer))
            else:
                ret.append(token)
        return ''.join(ret)

def pickOptions(s):
    return OptionList().tokenize(s)

conf.registerPlugin('MoobotFactoids')
conf.registerChannelValue(conf.supybot.plugins.MoobotFactoids,
    'showFactoidIfOnlyOneMatch', registry.Boolean(True, """Determines whether
    or not the factoid value will be shown when a listkeys search returns only
    one factoid key."""))

class MoobotDBHandler(plugins.DBHandler):
    def makeDb(self, filename):
        """create MoobotFactoids database and tables"""
        if os.path.exists(filename):
            db = sqlite.connect(filename)
        else:
            db = sqlite.connect(filename, converters={'bool': bool})
            cursor = db.cursor()
            cursor.execute("""CREATE TABLE factoids (
                              key TEXT PRIMARY KEY,
                              created_by INTEGER,
                              created_at TIMESTAMP,
                              modified_by INTEGER,
                              modified_at TIMESTAMP,
                              locked_at TIMESTAMP,
                              locked_by INTEGER,
                              last_requested_by TEXT,
                              last_requested_at TIMESTAMP,
                              fact TEXT,
                              requested_count INTEGER
                              )""")
            db.commit()
        return db

    
class MoobotFactoids(callbacks.PrivmsgCommandAndRegexp):
    priority = 98
    addressedRegexps = ['changeFactoid', 'augmentFactoid',
                        'replaceFactoid', 'addFactoid']
    def __init__(self):
        callbacks.PrivmsgCommandAndRegexp.__init__(self)
        self.dbHandler = MoobotDBHandler(dbfilename)

    def die(self):
        # Handle DB stuff
        self.dbHandler.die()

    def _parseFactoid(self, irc, msg, fact):
        type = "define"  # Default is to just spit the factoid back as a
                         # definition of what the key is (i.e., "foo is bar")
        newfact = pickOptions(fact)
        if newfact.startswith("<reply>"):
            newfact = newfact[7:]
            type = "reply"
        elif newfact.startswith("<action>"):
            newfact = newfact[8:]
            type = "action"
        newfact = newfact.strip()
        newfact = plugins.standardSubstitute(irc, msg, newfact)
        return (type, newfact)

    def updateFactoidRequest(self, key, hostmask):
        """Updates the last_requested_* fields of a factoid row"""
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""UPDATE factoids SET
                          last_requested_by = %s,
                          last_requested_at = %s,
                          requested_count = requested_count +1
                          WHERE key = %s""",
                          hostmask, int(time.time()), key)
        db.commit()

    def randomfactoid(self, irc, msg, args):
        """<takes no arguments>

        Displays a random factoid (along with its key) from the database.
        """
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT fact, key FROM factoids 
                          ORDER BY random() LIMIT 1""")
        if cursor.rowcount == 0:
            irc.error('No factoids in the database.')
            return
        (fact, key) = cursor.fetchone()
        irc.reply('"%s" is "%s"' % (key, fact))


    def invalidCommand(self, irc, msg, tokens):
        key = ' '.join(tokens)
        key = key.rstrip('?!')
        if key.startswith('\x01'):
            return
        # Check the factoid db for an appropriate reply
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT fact FROM factoids WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            return False
        else:
            fact = cursor.fetchone()[0]
            # Update the requested count/requested by for this key
            hostmask = msg.prefix
            self.updateFactoidRequest(key, hostmask)
            # Now actually get the factoid and respond accordingly
            (type, text) = self._parseFactoid(irc, msg, fact)
            if type == "action":
                irc.reply(text, action=True)
            elif type == "reply":
                irc.reply(text, prefixName=False)
            elif type == "define":
                irc.reply("%s is %s" % (key, text), prefixName=False)
            else:
                irc.error("Spurious type from _parseFactoid.")
            return True

    def addFactoid(self, irc, msg, match):
        r"^(?!\x01)(.+?)\s+(?:is|_is_)\s+(.+)"
        # Check and see if there is a command that matches this that didn't
        # get caught due to nesting
#        cb = callbacks.findCallbackForCommand(irc, msg)
#        if cb:
#            irc.reply(irc.getHelp(cb[0].config))
#            return
        # First, check and see if the entire message matches a factoid key
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        key = match.group().rstrip('?! ')
        cursor.execute("""SELECT * FROM factoids WHERE key LIKE %s""", key)
        if cursor.rowcount != 0:
            self.invalidCommand(irc, msg, callbacks.tokenize(match.group()))
            return
        # Okay, we are REALLY adding stuff
        # Must be registered!
        try:
            id = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        key, fact = match.groups()
        # These are okay, unless there's an _is_ in there, in which case
        # we split on the leftmost one.
        if '_is_' in match.group():
            key, fact = imap(str.strip, match.group().split('_is_', 1))
        # Strip the key of punctuation and spaces
        key = key.rstrip('?! ')
        # Check and make sure it's not in the DB already
        cursor.execute("""SELECT * FROM factoids WHERE key LIKE %s""", key)
        if cursor.rowcount != 0:
            irc.error('Factoid "%s" already exists.' % key)
            return
        # Otherwise, 
        cursor.execute("""INSERT INTO factoids VALUES
                          (%s, %s, %s, NULL, NULL, NULL, NULL, NULL, NULL,
                           %s, 0)""",
                           key, id, int(time.time()), fact)
        db.commit()
        irc.replySuccess()

    def changeFactoid(self, irc, msg, match):
        r"(.+)\s+=~\s+(.+)"
        # Must be registered!
        try:
            id = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        key, regexp = match.groups()
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        # Check and make sure it's in the DB 
        cursor.execute("""SELECT locked_at, fact FROM factoids
                          WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('Factoid "%s" not found.' % key)
            return
        # No dice if it's locked, no matter who it is
        (locked_at, fact) = cursor.fetchone()
        if locked_at is not None:
            irc.error('Factoid "%s" is locked.' % key)
            return
        # It's fair game if we get to here
        try:
            r = utils.perlReToReplacer(regexp)
        except ValueError, e:
            irc.error('Invalid regexp: "%s"' % regexp)
            return
        new_fact = r(fact) 
        cursor.execute("""UPDATE factoids   
                          SET fact = %s, modified_by = %s,   
                          modified_at = %s WHERE key = %s""",
                          new_fact, id, int(time.time()), key)
        db.commit()
        irc.replySuccess()

    def augmentFactoid(self, irc, msg, match):
        r"(.+?) is also (.+)"
        # Must be registered!
        try:
            id = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        key, new_text = match.groups()
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        # Check and make sure it's in the DB 
        cursor.execute("""SELECT locked_at, fact FROM factoids
                          WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('Factoid "%s" not found.' % key)
            return
        # No dice if it's locked, no matter who it is
        (locked_at, fact) = cursor.fetchone()
        if locked_at is not None:
            irc.error('Factoid "%s" is locked.' % key)
            return
        # It's fair game if we get to here
        new_fact = "%s, or %s" % (fact, new_text)
        cursor.execute("""UPDATE factoids
                          SET fact = %s, modified_by = %s,
                          modified_at = %s WHERE key = %s""",
                          new_fact, id, int(time.time()), key)
        db.commit()
        irc.replySuccess()

    def replaceFactoid(self, irc, msg, match):
        r"^no,?\s+(.+?)\s+is\s+(.+)"
        # Must be registered!
        try:
            id = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        key, new_fact = match.groups()
        # These are okay, unless there's an _is_ in there, in which case
        # we split on the leftmost one.
        if '_is_' in match.group():
            key, new_fact = imap(str.strip, match.group().split('_is_', 1))
            key = key.split(' ', 1)[1]  # Take out everything to first space
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        # Check and make sure it's in the DB 
        cursor.execute("""SELECT locked_at, fact FROM factoids
                          WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('Factoid "%s" not found.' % key)
            return
        # No dice if it's locked, no matter who it is
        (locked_at, _) = cursor.fetchone()
        if locked_at is not None:
            irc.error('Factoid "%s" is locked.' % key)
            return
        # It's fair game if we get to here
        cursor.execute("""UPDATE factoids
                          SET fact = %s, created_by = %s,
                          created_at = %s, modified_by = NULL,
                          modified_at = NULL, requested_count = 0,
                          last_requested_by = NULL, last_requested_at = NULL,
                          locked_at = NULL
                          WHERE key = %s""",
                          new_fact, id, int(time.time()), key)
        db.commit()
        irc.replySuccess()

    def literal(self, irc, msg, args):
        """<factoid key>

        Returns the literal factoid for the given factoid key.  No parsing of
        the factoid value is done as it is with normal retrieval.
        """
        key = privmsgs.getArgs(args, required=1)
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT fact FROM factoids WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('No such factoid: "%s"' % key)
            return
        else:
            fact = cursor.fetchone()[0]
            irc.reply(fact)

    def factinfo(self, irc, msg, args):
        """<factoid key>

        Returns the various bits of info on the factoid for the given key.
        """
        key = privmsgs.getArgs(args, required=1)
        # Start building the response string
        s = key + ": "
        # Next, get all the info and build the response piece by piece
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT created_by, created_at, modified_by,
                          modified_at, last_requested_by, last_requested_at,
                          requested_count, locked_by, locked_at FROM
                          factoids WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('No such factoid: "%s"' % key)
            return
        (created_by, created_at, modified_by, modified_at, last_requested_by,
         last_requested_at, requested_count, locked_by,
         locked_at) = cursor.fetchone()
        # First, creation info.
        # Map the integer created_by to the username
        creat_by = ircdb.users.getUser(created_by).name
        creat_at = time.strftime(conf.supybot.humanTimestampFormat(),
                                 time.localtime(int(created_at)))
        s += "Created by %s on %s." % (creat_by, creat_at)
        # Next, modification info, if any.
        if modified_by is not None:
            mod_by = ircdb.users.getUser(modified_by).name
            mod_at = time.strftime(conf.supybot.humanTimestampFormat(),
                                   time.localtime(int(modified_at)))
            s += " Last modified by %s on %s." % (mod_by, mod_at)
        # Next, last requested info, if any
        if last_requested_by is not None:
            last_by = last_requested_by  # not an int user id
            last_at = time.strftime(conf.supybot.humanTimestampFormat(),
                                    time.localtime(int(last_requested_at)))
            req_count = requested_count
            times_str = utils.nItems('time', requested_count)
            s += " Last requested by %s on %s, requested %s." % \
                 (last_by, last_at, times_str)
        # Last, locked info
        if locked_at is not None:
            lock_at = time.strftime(conf.supybot.humanTimestampFormat(),
                                     time.localtime(int(locked_at)))
            lock_by = ircdb.users.getUser(locked_by).name
            s += " Locked by %s on %s." % (lock_by, lock_at)
        irc.reply(s)

    def _lock(self, irc, msg, args, locking=True):
        self.log.debug('in _lock')
        try:
            id = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        self.log.debug('id: %s' % id)
        key = privmsgs.getArgs(args, required=1)
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT created_by, locked_by FROM factoids
                          WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('No such factoid: "%s"' % key)
            return
        (created_by, locked_by) = cursor.fetchone()
        # Don't perform redundant operations
        if locking and locked_by is not None:
               irc.error('Factoid "%s" is already locked.' % key)
               return
        if not locking and locked_by is None:
               irc.error('Factoid "%s" is not locked.' % key)
               return
        # Can only lock/unlock own factoids unless you're an admin
        self.log.debug('admin?: %s' % ircdb.checkCapability(id, 'admin'))
        self.log.debug('created_by: %s' % created_by)
        if not (ircdb.checkCapability(id, 'admin') or created_by == id):
            if locking:
                s = "lock"
            else:
                s = "unlock"
            irc.error("Cannot %s someone else's factoid unless you "
                           "are an admin." % s)
            return
        # Okay, we're done, ready to lock/unlock
        if locking:
           locked_at = int(time.time())
        else:
           locked_at = None
           id = None
        cursor.execute("""UPDATE factoids SET locked_at = %s, locked_by = %s
                          WHERE key = %s""", locked_at, id, key)
        db.commit()
        irc.replySuccess()

    def lock(self, irc, msg, args):
        """<factoid key>

        Locks the factoid with the given factoid key.  Requires that the user
        be registered and have created the factoid originally.
        """
        self._lock(irc, msg, args, True)

    def unlock(self, irc, msg, args):
        """<factoid key>

        Unlocks the factoid with the given factoid key.  Requires that the
        user be registered and have locked the factoid.
        """
        self._lock(irc, msg, args, False)

    _mostCount = 10
    class MostException(Exception):
        pass
    def most(self, irc, msg, args):
        """<popular|authored|recent>

        Lists the most <popular|authored|recent> factoids.  <popular> lists the
        most frequently requested factoids.  <authored> lists the author with
        the most factoids.  <recent> lists the most recently created factoids.
        """
        arg = privmsgs.getArgs(args)
        arg = arg.capitalize()
        method = getattr(self, '_most%s' % arg, None)
        if method is None:
            raise callbacks.ArgumentError
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT COUNT(*) FROM factoids""")
        if int(cursor.fetchone()[0]) == 0:
            irc.error('I don\'t have any factoids in my database!')
        else:
            try:
                irc.reply(method(cursor, self._mostCount))
            except self.MostException, e:
                irc.error(str(e))

    def _mostAuthored(self, cursor, limit):
        cursor.execute("""SELECT created_by, count(key) FROM factoids
                          GROUP BY created_by
                          ORDER BY count(key) DESC LIMIT %s""", limit)
        L = ['%s (%s)' % (ircdb.users.getUser(t[0]).name, int(t[1]))
             for t in cursor.fetchall()]
        return 'Most prolific %s: %s' % \
               (utils.pluralize('author', len(L)), utils.commaAndify(L))

    def _mostRecent(self, cursor, limit):
        cursor.execute("""SELECT key FROM factoids
                          ORDER by created_at DESC LIMIT %s""", limit)
        L = ['"%s"' % t[0] for t in cursor.fetchall()]
        return '%s: %s' % \
               (utils.nItems('factoid', len(L), between='latest'),
                utils.commaAndify(L))

    def _mostPopular(self, cursor, limit):
        cursor.execute("""SELECT key, requested_count FROM factoids
                          WHERE requested_count > 0
                          ORDER BY requested_count DESC LIMIT %s""", limit)
        if cursor.rowcount == 0:
            raise self.MostException, 'No factoids have been requested.'
        L = ['"%s" (%s)' % (t[0], t[1]) for t in cursor.fetchall()]
        return 'Top %s: %s' % \
               (utils.nItems('factoid', len(L), between='requested'),
                utils.commaAndify(L))

    def listauth(self, irc, msg, args):
        """<author name>

        Lists the keys of the factoids with the given author.  Note that if an
        author has an integer name, you'll have to use that author's id to use
        this function (so don't use integer usernames!).
        """
        author = privmsgs.getArgs(args, required=1)
        try:
            id = ircdb.users.getUserId(author)
        except KeyError:
            irc.error("No such user: %r" % author)
            return
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT key FROM factoids
                          WHERE created_by = %s
                          ORDER BY key""", id)
        if cursor.rowcount == 0:
            irc.reply('No factoids by "%s" found.' % author)
            return
        keys = ['"%s"' % tup[0] for tup in cursor.fetchall()]
        s = 'Author search for "%s" (%s found): %s' % \
            (author, len(keys), utils.commaAndify(keys))
        irc.reply(s)

    def listkeys(self, irc, msg, args):
        """<text>

        Lists the keys of the factoids whose key contains the provided text.
        """
        search = privmsgs.getArgs(args, required=1)
        # Don't error if we aren't in a channel, private messages are okay
        channel = privmsgs.getChannel(msg, args, raiseError=False)
        glob = '%' + search + '%'
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT key FROM factoids
                          WHERE key LIKE %s
                          ORDER BY key""",
                          glob)
        if cursor.rowcount == 0:
            irc.reply('No keys matching "%s" found.' % search)
        elif cursor.rowcount == 1 and \
        self.registryValue('showFactoidIfOnlyOneMatch', channel):
            key = cursor.fetchone()[0]
            self.invalidCommand(irc, msg, [key])
        else:
            keys = ['"%s"' % tup[0] for tup in cursor.fetchall()]
            s = 'Key search for "%s" (%s found): %s' % \
                (search, len(keys), utils.commaAndify(keys))
            irc.reply(s)

    def listvalues(self, irc, msg, args):
        """<text>

        Lists the keys of the factoids whose value contains the provided text.
        """
        search = privmsgs.getArgs(args, required=1)
        glob = '%' + search + '%'
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT key FROM factoids
                          WHERE fact LIKE %s
                          ORDER BY key""",
                          glob)
        if cursor.rowcount == 0:
            irc.reply('No values matching "%s" found.' % search)
            return
        keys = ['"%s"' % tup[0] for tup in cursor.fetchall()]
        s = 'Value search for "%s" (%s found): %s' % \
            (search, len(keys), utils.commaAndify(keys))
        irc.reply(s)

    def delete(self, irc, msg, args):
        """<factoid key>

        Deletes the factoid with the given key.
        """
        # Must be registered to use this
        try:
            ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        key = privmsgs.getArgs(args, required=1)
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT key, locked_at FROM factoids
                          WHERE key LIKE %s""", key)
        if cursor.rowcount == 0:
            irc.error('No such factoid: "%s"' % key)
            return
        (_, locked_at) = cursor.fetchone() 
        if locked_at is not None:
            irc.error("Factoid is locked, cannot remove.")
            return
        cursor.execute("""DELETE FROM factoids WHERE key = %s""", key)
        db.commit()
        irc.replySuccess()

    def randomfactoid(self, irc, msg, args):
        """<takes no arguments>

        Displays a random factoid (along with its key) from the database.
        """
        db = self.dbHandler.getDb()
        cursor = db.cursor()
        cursor.execute("""SELECT fact, key FROM factoids 
                          ORDER BY random() LIMIT 1""")
        if cursor.rowcount == 0:
            irc.error('No factoids in the database.')
            return
        (fact, key) = cursor.fetchone()
        irc.reply('Random factoid: "%s" is "%s"' % (key, fact))

Class = MoobotFactoids

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Removes all bold output by the bot.
"""

__revision__ = "$Id: KillBold.py,v 1.8 2004/01/29 23:58:28 jamessan Exp $"

import plugins

import re

import ircmsgs
import callbacks


# For some stupid reason, this doesn't work.
## boldre = re.compile('(?:\x02([^\x02\x03\x0f]*\x03))|'
##                     '(?:\x02([^\x02\x0f]*)(?:[\x02\x0f]|$))')
boldre1 = re.compile('(?:\x02([^\x02\x03\x0f]*\x03))')
boldre2 = re.compile('(?:\x02([^\x02\x0f]*)(?:[\x02\x0f]|$))')

class KillBold(callbacks.Privmsg):
    priority = 10
    def outFilter(self, irc, msg):
        if msg.command == 'PRIVMSG':
            s = boldre1.sub(r'\1', msg.args[1])
            s = boldre2.sub(r'\1', s)
            return ircmsgs.privmsg(msg.args[0], s)
        else:
            return msg

Class = KillBold

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python
# -*- coding:utf-8 -*-

###
# Copyright (c) 2002, Stéphan Kochen
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
A module for managing and voting on polls.
"""

__revision__ = "$Id: Poll.py,v 1.24 2004/04/15 06:41:04 jemfinch Exp $"

import plugins

import os
import time

import conf
import utils
import ircdb
import ircutils
import privmsgs
import callbacks

try:
    import sqlite
except ImportError:
    raise callbacks.Error, 'You need to have PySQLite installed to use this ' \
                           'plugin.  Download it at <http://pysqlite.sf.net/>'

class Poll(callbacks.Privmsg, plugins.ChannelDBHandler):
    def __init__(self):
        callbacks.Privmsg.__init__(self)
        plugins.ChannelDBHandler.__init__(self)

    def makeDb(self, filename):
        if os.path.exists(filename):
            db = sqlite.connect(filename)
        else:
            db = sqlite.connect(filename)
            cursor = db.cursor()
            cursor.execute("""CREATE TABLE polls (
                              id INTEGER PRIMARY KEY,
                              question TEXT UNIQUE ON CONFLICT IGNORE,
                              started_by INTEGER,
                              open INTEGER)""")
            cursor.execute("""CREATE TABLE options (
                              id INTEGER,
                              poll_id INTEGER,
                              option TEXT,
                              UNIQUE (poll_id, id) ON CONFLICT IGNORE)""")
            cursor.execute("""CREATE TABLE votes (
                              user_id INTEGER,
                              poll_id INTEGER,
                              option_id INTEGER,
                              UNIQUE (user_id, poll_id)
                              ON CONFLICT IGNORE)""")
            db.commit()
        return db

    def poll(self, irc, msg, args):
        """<id>

        Displays the poll question and options for the given poll id.
        """
        channel = privmsgs.getChannel(msg, args)
        poll_id = privmsgs.getArgs(args)
        db = self.getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT id, question, started_by, open
                          FROM polls WHERE id=%s""",
                          poll_id)
        if cursor.rowcount == 0:
            irc.error('There is no poll with id %s' % poll_id)
            return
        _, question, started_by, open = cursor.fetchone()
        starter = ircdb.users.getUser(started_by).name
        if open:
            statusstr = 'open'
        else:
            statusstr = 'closed'
        cursor.execute("""SELECT id, option FROM options
                          WHERE poll_id=%s""",
                          poll_id)
        if cursor.rowcount == 0:
            optionstr = 'This poll has no options yet'
        else:
            options = cursor.fetchall()
            optionstr = 'Options:'
            optionstr += ''.join([' %s: %r' % (id, option)
                                 for id, option in options])
        pollstr = 'Poll #%s: %r started by %s. %s. Poll is %s.' % \
                  (poll_id, question, starter, optionstr, statusstr)
        irc.reply(pollstr)

    def open(self, irc, msg, args):
        """[<channel>] <question>
        
        Creates a new poll with the given question.
        """
        channel = privmsgs.getChannel(msg, args)
        question = privmsgs.getArgs(args)
        # Must be registered to create a poll
        try:
            userId = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        db = self.getDb(channel)
        cursor = db.cursor()
        cursor.execute("""INSERT INTO polls
                          VALUES (NULL, %s, %s, 1)""",
                          question, userId)
        db.commit()
        cursor.execute("""SELECT id FROM polls WHERE question=%s""", question)
        id = cursor.fetchone()[0]
        irc.replySuccess('(poll #%s added)' % id)

    def close(self, irc, msg, args):
        """[<channel>] <id>
        
        Closes the poll with the given <id>; further votes will not be allowed.
        """
        channel = privmsgs.getChannel(msg, args)
        id = privmsgs.getArgs(args)
        try:
            id = int(id)
        except ValueError:
            irc.error('The id must be an integer.')
            return
        db = self.getDb(channel)
        cursor = db.cursor()
        # Check to make sure that the poll exists
        cursor.execute("""SELECT id FROM polls WHERE id=%s""", id)
        if cursor.rowcount == 0:
            irc.error('Id #%s is not an existing poll.')
            return
        cursor.execute("""UPDATE polls SET open=0 WHERE id=%s""", id)
        irc.replySuccess()

    def add(self, irc, msg, args):
        """[<channel>] <id> <option text>
        
        Add an option with the given text to the poll with the given id.
        """
        channel = privmsgs.getChannel(msg, args)
        (poll_id, option) = privmsgs.getArgs(args, required=2)
        try:
            poll_id = int(poll_id)
        except ValueError:
            irc.error('The id must be an integer.')
            return
        try:
            userId = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        db = self.getDb(channel)
        cursor = db.cursor()
        # Only the poll starter or an admin can add options
        cursor.execute("""SELECT started_by FROM polls
                          WHERE id=%s""",
                          poll_id)
        if cursor.rowcount == 0:
            irc.error('There is no such poll.')
            return
        if not ((userId == cursor.fetchone()[0]) or
                (ircdb.checkCapability(userId, 'admin'))):
            irc.error('That poll isn\'t yours and you aren\'t an admin.')
            return
        # and NOBODY can add options once a poll has votes
        cursor.execute("""SELECT COUNT(user_id) FROM votes
                          WHERE poll_id=%s""",
                          poll_id)
        if int(cursor.fetchone()[0]) != 0:
            irc.error('Cannot add options to a poll with votes.')
            return
        # Get the next highest id
        cursor.execute("""SELECT MAX(id)+1 FROM options
                          WHERE poll_id=%s""",
                          poll_id)
        option_id = cursor.fetchone()[0] or 1
        cursor.execute("""INSERT INTO options VALUES
                          (%s, %s, %s)""",
                          option_id, poll_id, option)
        irc.replySuccess()

    def vote(self, irc, msg, args):
        """[<channel>] <poll id> <option id>
        
        Vote for the option with the given id on the poll with the given poll
        id.  This command can also be used to override any previous vote.
        """
        channel = privmsgs.getChannel(msg, args)
        (poll_id, option_id) = privmsgs.getArgs(args, required=2)
        try:
            poll_id = int(poll_id)
            option_id = int(option_id)
        except ValueError:
            irc.error('The poll id and option id '
                           'arguments must be an integers.')
            return
        try:
            userId = ircdb.users.getUserId(msg.prefix)
        except KeyError:
            irc.errorNotRegistered()
            return
        db = self.getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT open
                          FROM polls WHERE id=%s""",
                          poll_id)
        if cursor.rowcount == 0:
            irc.error('There is no such poll.')
            return
        elif int(cursor.fetchone()[0]) == 0:
            irc.error('That poll is closed.')
            return
        cursor.execute("""SELECT id FROM options
                          WHERE poll_id=%s
                          AND id=%s""",
                          poll_id, option_id)
        if cursor.rowcount == 0:
            irc.error('There is no such option.')
            return
        cursor.execute("""SELECT option_id FROM votes
                          WHERE user_id=%s AND poll_id=%s""",
                          userId, poll_id)
        if cursor.rowcount == 0:
            cursor.execute("""INSERT INTO votes VALUES (%s, %s, %s)""",
                           userId, poll_id, option_id)
        else:
            cursor.execute("""UPDATE votes SET option_id=%s
                              WHERE user_id=%s AND poll_id=%s""",
                              option_id, userId, poll_id)
        irc.replySuccess()

    def results(self, irc, msg, args):
        """[<channel>] <id>
        
        Shows the results for the poll with the given id.
        """
        channel = privmsgs.getChannel(msg, args)
        poll_id = privmsgs.getArgs(args)
        try:
            poll_id = int(poll_id)
        except ValueError:
            irc.error('The id argument must be an integer.')
            return
        db = self.getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT id, question, started_by, open
                          FROM polls WHERE id=%s""",
                          poll_id)
        if cursor.rowcount == 0:
            irc.error('There is no such poll.')
            return
        (id, question, startedBy, open) = cursor.fetchone()
        try:
            startedBy = ircdb.users.getUser(startedBy).name
        except KeyError:
            startedBy = 'an unknown user'
            return
        reply = 'Results for poll #%s: "%s" by %s' % \
                (id, question, startedBy)
        cursor.execute("""SELECT count(user_id), option_id
                          FROM votes
                          WHERE poll_id=%s 
                          GROUP BY option_id
                          UNION
                          SELECT 0, id AS option_id
                          FROM options
                          WHERE poll_id=%s
                          AND id NOT IN (
                                SELECT option_id FROM votes
                                WHERE poll_id=%s)
                          GROUP BY option_id
                          ORDER BY count(user_id) DESC""",
                          poll_id, poll_id, poll_id)
        if cursor.rowcount == 0:
            s = 'This poll has no votes yet.'
        else:
            results = []
            for count, option_id in cursor.fetchall():
                cursor.execute("""SELECT option FROM options
                                  WHERE id=%s AND poll_id=%s""",
                                  option_id, poll_id)
                option = cursor.fetchone()[0]
                results.append('%r: %s' % (option, int(count)))
            s = utils.commaAndify(results)
        reply += ' - %s' % s
        irc.reply(reply)
    
    def list(self, irc, msg, args):
        """takes no arguments.

        Lists the currently open polls for the channel and their ids.
        """
        channel = privmsgs.getChannel(msg, args)
        db = self.getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT id, question FROM polls WHERE open=1""")
        if cursor.rowcount == 0:
            irc.reply('This channel currently has no open polls.')
            return
        polls = ['#%s: %r' % (id, q) for id, q in cursor.fetchall()]
        irc.reply(utils.commaAndify(polls))

Class = Poll

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:



#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Supports various DCC things.
"""

__revision__ = "$Id: DCC.py,v 1.12 2004/01/29 23:58:27 jamessan Exp $"

import plugins

import socket
import textwrap
import threading

import conf
import utils
import world
import ircmsgs
import ircutils
import privmsgs
import callbacks

class DCC(callbacks.Privmsg):
    def chat(self, irc, msg, args):
        """<text>

        Sends <text> to the user via a DCC CHAT.  Use nested commands to your
        benefit here.
        """
        text = privmsgs.getArgs(args)
        def openChatPort():
            try:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.settimeout(60)
                host = ircutils.hostFromHostmask(irc.prefix)
                if conf.supybot.externalIP():
                    ip = conf.supybot.externalIP()
                else:
                    try:
                        ip = socket.gethostbyname(host)
                    except socket.error, e:
                        s = 'Error trying to determine the external IP ' \
                            'address of this machine via the host %s: %s'
                        self.log.warning(s, host, e)
                        irc.replyError()
                        return
                i = ircutils.dccIP(ip)
                sock.bind((host, 0))
                port = sock.getsockname()[1]
                self.log.info('DCC CHAT port opened at (%s, %s)', host, port)
                sock.listen(1)
                irc.queueMsg(ircmsgs.privmsg(msg.nick,
                                             '\x01DCC CHAT chat %s %s\x01' % \
                                             (i, port)))
                (realSock, addr) = sock.accept()
                self.log.info('DCC CHAT accepted from %s', addr)
                for line in textwrap.wrap(text, 80):
                    realSock.send(line)
                    realSock.send('\n')
            finally:
                self.log.info('Finally closing sock and realSock.')
                sock.close()
                try:
                    realSock.close()
                except UnboundLocalError:
                    pass
        t = threading.Thread(target=openChatPort)
        world.threadsSpawned += 1
        t.setDaemon(True)
        t.start()
                         


Class = DCC

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
